-------------------------------------------------------
-- Design Name : feb_busy_test_01
-- File Name   : feb_busy_test_01.vhd

-- Function    : top level design, fanout combinatorial, 3nn
--               external 50 MHz or local 25 MHz clock, by jumper
-- Coder       : K.-H. Sulanke, DESY, 2024-06-19
-------------------------------------------------------

-- 2024-06-26	rev.01, trying to fix the blocked L1A-issue, in case of local clocking + DEBUG-rg=01h

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;

Library UNISIM;
use UNISIM.vcomponents.all;

entity feb_busy_test_01 is
  port
   ( 
    gclk2        : in std_logic; -- 50MHz
	 reset        : in std_logic;
	 trgl1_copy   : in std_logic;
	 trgl1_or     : in std_logic;
	 trgl1        : out std_logic;	 
	 debug_rg     : in std_logic_vector(7 downto 0); -- synchronous to gclk2
	 
    TEST_IO2_N   : in  std_logic; -- 
 
    busy_out     : in std_logic 

 );

end entity;

architecture feb_busy_test_01_arch of feb_busy_test_01 is
-- constant PPS_LENGTH          : integer          :=  3; -- in multiples of 20 ns

  signal busy_sync_pipe   : std_logic_vector(2 downto 0) := B"000";
  signal l1a_trig_pipe    : std_logic_vector(2 downto 0) := B"000";
  
  signal trgl1            : std_logic := '0'; -- lvds outbuf input, passed through from L2-crate
  signal trgl1_sync       : std_logic := '0';
  signal l1a_trig_lh_edge : std_logic := '0';
  signal l1a_trig_hl_edge : std_logic := '0';
  
  type state_type is (L1A_IDLE, L1A_RUN, L1A_BLOCKED);
  signal state : state_type := L1A_IDLE;
  
begin

-- I/O port mapping ----------------------------------------------------------------------------------

   BUSY_SYNC_GEN: process (gclk2, reset)
   begin
     if rising_edge (gclk2) then
--    if (reset = '1') or (((l1a_trig_scaler_wstb = '1') or (l1a_busy_trig_scaler_wstb = '1')) and (debug_rg = X"01" ))then
      if (reset = '1') then	  
		  busy_sync_pipe     <= B"000";
		else
        busy_sync_pipe(0)  <= busy_out; 
        busy_sync_pipe(1)  <= busy_sync_pipe(0);
		  busy_sync_pipe(2)  <= busy_sync_pipe(1);
      end if; -- if (reset = '1') or (l1a_trig_scaler_wstb = '1') 
     end if; -- rising_edge (gclk2)
    end process BUSY_SYNC_GEN;
	 
  L1A_EDGE_GEN: process (gclk2, reset)
   begin
     if rising_edge (gclk2) then
	   l1a_trig_lh_edge <= '0';	-- to get a single pulse
		l1a_trig_hl_edge <= '0';	-- to get a single pulse		
      if (reset = '1') then
		  l1a_trig_pipe     <= B"000";
		else
        l1a_trig_pipe(0)  <= trgl1_copy;  -- copy of the downgoing L1A trigger
        l1a_trig_pipe(1)  <= l1a_trig_pipe(0);
		  l1a_trig_pipe(2)  <= l1a_trig_pipe(1);  
	     if (l1a_trig_pipe = B"011") then 
         l1a_trig_lh_edge <= '1';
		  end if;
		  if (l1a_trig_pipe = B"100") then 
         l1a_trig_hl_edge <= '1';
		  end if;
      end if;
     end if;
   end process L1A_EDGE_GEN;
	
	 L1A_CONTROL: process (gclk2)
    begin
      if rising_edge (gclk2) then
			if (reset = '1')   then
			 state                 <= L1A_IDLE;
			 trgl1_sync            <= '0';
   		else
			 if ((debug_rg  = X"01") or (TEST_IO2_N ='0')) then  
				CAL_SM : 
				  case (state) is   
					 when L1A_IDLE => 
					   trgl1_sync             <= '0';
						if    busy_sync_pipe(2)='0' and l1a_trig_lh_edge <= '1' then		
						 state                 <= L1A_RUN;
						 trgl1_sync            <= '1';
						elsif busy_sync_pipe(2)='1' then		
						 state                 <= L1A_BLOCKED;
						end if;

					 when L1A_RUN => 
					   if l1a_trig_hl_edge <= '1' then trgl1_sync <= '0'; end if;					
						if    busy_sync_pipe(2) = '1' and l1a_trig_lh_edge <= '1' then		
						 state                 <= L1A_BLOCKED;
						elsif busy_sync_pipe(2) = '0' and l1a_trig_lh_edge <= '1' then
						 trgl1_sync            <= '1'; 
						end if; 

					 when L1A_BLOCKED => 
						if busy_sync_pipe(2)='0' and l1a_trig_lh_edge <= '1' then		
						 state                 <= L1A_RUN;
						 trgl1_sync            <= '1';
						end if;
					end case;				
				end if; --	((debug_rg  = X"01") or (TEST_IO2_N ='0')) 
			end if; -- (reset = '1')
     end if; -- rising_edge (gclk2)
    end process L1A_CONTROL;	
 
 TRGL1_GEN: process(trgl1_or, trgl1_sync, debug_rg, TEST_IO2_N)
   begin
	 if      (((debug_rg  = X"01") or  (TEST_IO2_N ='0')) and  trgl1_sync ='1')
        or  (((debug_rg /= X"01") and (TEST_IO2_N ='1')) and  trgl1_or   ='1')	 then
      trgl1      <= '1';
    else
      trgl1      <= '0';
	 end if;	 
   end process TRGL1_GEN; 
	
end architecture feb_busy_test_01;
	