-------------------------------------------------------
-- Design Name : iserdes_pulse_shaper_03
-- File Name   : iserdes_pulse_shaper_03.vhd
-- Device      : Spartan 6
-- Function    : modifying the iserdes output (progr. pulse length)
-- Coder       : K.-H. Sulanke, DESY, 2017-10-16
-------------------------------------------------------
-- 2017-03-06  "elsif" introduced
-- 2017-10-16  pulse_pdat_next 6 bit wide now, to remove compiler warning

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all ;

library unisim ;
use unisim.vcomponents.all ;

entity iserdes_pulse_shaper_03 is
  port 	(
         clk             :  in  std_logic ;                 -- pdat clock
         iserdes_pdat    :  in  std_logic_vector (0 to 7);  -- iserdes pdat out
         pulse_pdat      :  out std_logic_vector (0 to 7);  -- pulse shaped pdat
         pulse_width     :  in  std_logic_vector (2 downto 0)
        );
  end iserdes_pulse_shaper_03 ;

architecture arch_iserdes_pulse_shaper_03 of iserdes_pulse_shaper_03 is

  signal iserdes_pdat_p7  : std_logic :='0';  -- previous bit 7
  signal pulse_pdat_next  : std_logic_vector (0 to 5) := B"0000_00";
  
  type state_type is (PULSE_IDLE, PULSE_NEXT);
  signal state : state_type := PULSE_IDLE;

begin

  
PULSE_SHAPE: process (clk)
    begin
     if rising_edge(clk) then
      if pulse_width = B"000" then  -- no pulse shaping !
       pulse_pdat <= iserdes_pdat;
      else  -- start pulse shaping                  
      
       iserdes_pdat_p7 <= iserdes_pdat(7);
       
       case state is
       
        when PULSE_IDLE =>
 
          if pulse_width = B"001" then
           if    iserdes_pdat_p7='0' and iserdes_pdat(0)='1' then pulse_pdat <= X"80"; pulse_pdat_next <= B"0000_00";
           elsif iserdes_pdat(0)='0' and iserdes_pdat(1)='1' then pulse_pdat <= X"40"; pulse_pdat_next <= B"0000_00";
           elsif iserdes_pdat(1)='0' and iserdes_pdat(2)='1' then pulse_pdat <= X"20"; pulse_pdat_next <= B"0000_00";
           elsif iserdes_pdat(2)='0' and iserdes_pdat(3)='1' then pulse_pdat <= X"10"; pulse_pdat_next <= B"0000_00";
           elsif iserdes_pdat(3)='0' and iserdes_pdat(4)='1' then pulse_pdat <= X"08"; pulse_pdat_next <= B"0000_00";
           elsif iserdes_pdat(4)='0' and iserdes_pdat(5)='1' then pulse_pdat <= X"04"; pulse_pdat_next <= B"0000_00";
           elsif iserdes_pdat(5)='0' and iserdes_pdat(6)='1' then pulse_pdat <= X"02"; pulse_pdat_next <= B"0000_00";
           elsif iserdes_pdat(6)='0' and iserdes_pdat(7)='1' then pulse_pdat <= X"01"; pulse_pdat_next <= B"0000_00"; 
           end if;
          end if;
          
          if pulse_width = B"010" then
           if    iserdes_pdat_p7='0' and iserdes_pdat(0)='1' then pulse_pdat <= X"c0"; pulse_pdat_next <= B"0000_00";
           elsif iserdes_pdat(0)='0' and iserdes_pdat(1)='1' then pulse_pdat <= X"60"; pulse_pdat_next <= B"0000_00";
           elsif iserdes_pdat(1)='0' and iserdes_pdat(2)='1' then pulse_pdat <= X"30"; pulse_pdat_next <= B"0000_00";
           elsif iserdes_pdat(2)='0' and iserdes_pdat(3)='1' then pulse_pdat <= X"18"; pulse_pdat_next <= B"0000_00";
           elsif iserdes_pdat(3)='0' and iserdes_pdat(4)='1' then pulse_pdat <= X"0c"; pulse_pdat_next <= B"0000_00";
           elsif iserdes_pdat(4)='0' and iserdes_pdat(5)='1' then pulse_pdat <= X"06"; pulse_pdat_next <= B"0000_00";
           elsif iserdes_pdat(5)='0' and iserdes_pdat(6)='1' then pulse_pdat <= X"03"; pulse_pdat_next <= B"0000_00";
           elsif iserdes_pdat(6)='0' and iserdes_pdat(7)='1' then pulse_pdat <= X"01"; pulse_pdat_next <= B"1000_00"; 
           end if;
          end if;

          if pulse_width = B"011" then
           if    iserdes_pdat_p7='0' and iserdes_pdat(0)='1' then pulse_pdat <= X"e0"; pulse_pdat_next <= B"0000_00";
           elsif iserdes_pdat(0)='0' and iserdes_pdat(1)='1' then pulse_pdat <= X"70"; pulse_pdat_next <= B"0000_00";
           elsif iserdes_pdat(1)='0' and iserdes_pdat(2)='1' then pulse_pdat <= X"38"; pulse_pdat_next <= B"0000_00";
           elsif iserdes_pdat(2)='0' and iserdes_pdat(3)='1' then pulse_pdat <= X"1c"; pulse_pdat_next <= B"0000_00";
           elsif iserdes_pdat(3)='0' and iserdes_pdat(4)='1' then pulse_pdat <= X"0e"; pulse_pdat_next <= B"0000_00";
           elsif iserdes_pdat(4)='0' and iserdes_pdat(5)='1' then pulse_pdat <= X"07"; pulse_pdat_next <= B"0000_00";
           elsif iserdes_pdat(5)='0' and iserdes_pdat(6)='1' then pulse_pdat <= X"03"; pulse_pdat_next <= B"1000_00";
           elsif iserdes_pdat(6)='0' and iserdes_pdat(7)='1' then pulse_pdat <= X"01"; pulse_pdat_next <= B"1100_00"; 
           end if;
          end if;

          if pulse_width = B"100" then
           if    iserdes_pdat_p7='0' and iserdes_pdat(0)='1' then pulse_pdat <= X"f0"; pulse_pdat_next <= B"0000_00";
           elsif iserdes_pdat(0)='0' and iserdes_pdat(1)='1' then pulse_pdat <= X"78"; pulse_pdat_next <= B"0000_00";
           elsif iserdes_pdat(1)='0' and iserdes_pdat(2)='1' then pulse_pdat <= X"3c"; pulse_pdat_next <= B"0000_00";
           elsif iserdes_pdat(2)='0' and iserdes_pdat(3)='1' then pulse_pdat <= X"1e"; pulse_pdat_next <= B"0000_00";
           elsif iserdes_pdat(3)='0' and iserdes_pdat(4)='1' then pulse_pdat <= X"0f"; pulse_pdat_next <= B"0000_00";
           elsif iserdes_pdat(4)='0' and iserdes_pdat(5)='1' then pulse_pdat <= X"07"; pulse_pdat_next <= B"1000_00";
           elsif iserdes_pdat(5)='0' and iserdes_pdat(6)='1' then pulse_pdat <= X"03"; pulse_pdat_next <= B"1100_00";
           elsif iserdes_pdat(6)='0' and iserdes_pdat(7)='1' then pulse_pdat <= X"01"; pulse_pdat_next <= B"1110_00"; 
           end if;
          end if;

          if pulse_width = B"101" then
           if    iserdes_pdat_p7='0' and iserdes_pdat(0)='1' then pulse_pdat <= X"f8"; pulse_pdat_next <= B"0000_00";
           elsif iserdes_pdat(0)='0' and iserdes_pdat(1)='1' then pulse_pdat <= X"7c"; pulse_pdat_next <= B"0000_00";
           elsif iserdes_pdat(1)='0' and iserdes_pdat(2)='1' then pulse_pdat <= X"3e"; pulse_pdat_next <= B"0000_00";
           elsif iserdes_pdat(2)='0' and iserdes_pdat(3)='1' then pulse_pdat <= X"1f"; pulse_pdat_next <= B"0000_00";
           elsif iserdes_pdat(3)='0' and iserdes_pdat(4)='1' then pulse_pdat <= X"0f"; pulse_pdat_next <= B"1000_00";
           elsif iserdes_pdat(4)='0' and iserdes_pdat(5)='1' then pulse_pdat <= X"07"; pulse_pdat_next <= B"1100_00";
           elsif iserdes_pdat(5)='0' and iserdes_pdat(6)='1' then pulse_pdat <= X"03"; pulse_pdat_next <= B"1110_00";
           elsif iserdes_pdat(6)='0' and iserdes_pdat(7)='1' then pulse_pdat <= X"01"; pulse_pdat_next <= B"1111_00"; 
           end if;
          end if;
 
          if pulse_width = B"110" then
           if    iserdes_pdat_p7='0' and iserdes_pdat(0)='1' then pulse_pdat <= X"fc"; pulse_pdat_next <= B"0000_00";
           elsif iserdes_pdat(0)='0' and iserdes_pdat(1)='1' then pulse_pdat <= X"7e"; pulse_pdat_next <= B"0000_00";
           elsif iserdes_pdat(1)='0' and iserdes_pdat(2)='1' then pulse_pdat <= X"3f"; pulse_pdat_next <= B"0000_00";
           elsif iserdes_pdat(2)='0' and iserdes_pdat(3)='1' then pulse_pdat <= X"1f"; pulse_pdat_next <= B"1000_00";
           elsif iserdes_pdat(3)='0' and iserdes_pdat(4)='1' then pulse_pdat <= X"0f"; pulse_pdat_next <= B"1100_00";
           elsif iserdes_pdat(4)='0' and iserdes_pdat(5)='1' then pulse_pdat <= X"07"; pulse_pdat_next <= B"1110_00";
           elsif iserdes_pdat(5)='0' and iserdes_pdat(6)='1' then pulse_pdat <= X"03"; pulse_pdat_next <= B"1111_00";
           elsif iserdes_pdat(6)='0' and iserdes_pdat(7)='1' then pulse_pdat <= X"01"; pulse_pdat_next <= B"1111_10";
           end if;
          end if;

          if pulse_width = B"111" then
           if    iserdes_pdat_p7='0' and iserdes_pdat(0)='1' then pulse_pdat <= X"fe"; pulse_pdat_next <= B"0000_00";
           elsif iserdes_pdat(0)='0' and iserdes_pdat(1)='1' then pulse_pdat <= X"7f"; pulse_pdat_next <= B"0000_00";
           elsif iserdes_pdat(1)='0' and iserdes_pdat(2)='1' then pulse_pdat <= X"3f"; pulse_pdat_next <= B"1000_00";
           elsif iserdes_pdat(2)='0' and iserdes_pdat(3)='1' then pulse_pdat <= X"1f"; pulse_pdat_next <= B"1100_00";
           elsif iserdes_pdat(3)='0' and iserdes_pdat(4)='1' then pulse_pdat <= X"0f"; pulse_pdat_next <= B"1110_00";
           elsif iserdes_pdat(4)='0' and iserdes_pdat(5)='1' then pulse_pdat <= X"07"; pulse_pdat_next <= B"1111_00";
           elsif iserdes_pdat(5)='0' and iserdes_pdat(6)='1' then pulse_pdat <= X"03"; pulse_pdat_next <= B"1111_10";
           elsif iserdes_pdat(6)='0' and iserdes_pdat(7)='1' then pulse_pdat <= X"01"; pulse_pdat_next <= B"1111_11"; 
           end if;
          end if;
        
          if iserdes_pdat /= X"00" then
           state <= PULSE_NEXT;
          else
           pulse_pdat      <= X"00";
           pulse_pdat_next <= B"0000_00";
          end if; 
          
        when PULSE_NEXT =>  
           pulse_pdat <= (pulse_pdat_next & B"00");
           state      <= PULSE_IDLE;
           
       end case; -- state machine
      end if; --  if pulse_width = B"000"
     end if; -- if rising_edge(clk)
    end process PULSE_SHAPE; 
   

end arch_iserdes_pulse_shaper_03;