-------------------------------------------------------
-- Design Name : oserdes_01
-- File Name   : oserdes_01.vhd
-- Device      : Spartan 6
-- Function    : generating oserdes output from 8-bit par. data
-- Coder       : K.-H. Sulanke, DESY, 2017-10-23
-------------------------------------------------------
-- 2017-10-23  first revision

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all ;

library unisim ;
use unisim.vcomponents.all ;

entity oserdes_01 is
  port 	(
         clk             :  in  std_logic ;                    -- pdat clock
         reset           :  in  std_logic;
         oserdes_pdat    :  in  std_logic_vector (0 to 7);     -- serdes pdat
         oserdes_ioclk   :  in  std_logic;
         oserdes_stb     :  in  std_logic;
         oserdes_out     :  out std_logic   -- serial oserdes output
        );
  end oserdes_01 ;

architecture arch_oserdes_01 of oserdes_01 is
  
  signal shiftout3_s : std_logic;
  signal shiftout4_s : std_logic;
  signal shiftout1_m : std_logic;
  signal shiftout2_m : std_logic;
 
begin

    OSERDES_bank0_m: OSERDES2
       generic map (
         DATA_RATE_OQ   => "SDR",
         DATA_RATE_OT   => "SDR",
         TRAIN_PATTERN  => 0,
         DATA_WIDTH     => 8,
         SERDES_MODE    => "MASTER",
         OUTPUT_MODE    => "SINGLE_ENDED") --"DIFFERENTIAL")--
       port map (
        D1         => oserdes_pdat(4),
        D2         => oserdes_pdat(5),
        D3         => oserdes_pdat(6),
        D4         => oserdes_pdat(7),
        T1         => '0',
        T2         => '0',
        T3         => '0',
        T4         => '0',
        SHIFTIN1   => '1',
        SHIFTIN2   => '1',
        SHIFTIN3   => shiftout3_s, --ocascade_sm_d(pin_count),
        SHIFTIN4   => shiftout4_s, --ocascade_sm_t(pin_count),
        SHIFTOUT1  => shiftout1_m, --ocascade_ms_d(pin_count),
        SHIFTOUT2  => shiftout2_m, --ocascade_ms_t(pin_count),
        SHIFTOUT3  => open,
        SHIFTOUT4  => open,
        TRAIN      => '0',
        OCE        => '1',--clock_enable,
        CLK0       => oserdes_ioclk,
        CLK1       => '0',
        CLKDIV     => clk,
        OQ         => oserdes_out,
        TQ         => open,
        IOCE       => oserdes_stb,
        TCE        => '1',
        RST        => reset);

   OSERDES_bank0_s : OSERDES2
       generic map (
         DATA_RATE_OQ   => "SDR",
         DATA_RATE_OT   => "SDR",
         DATA_WIDTH     => 8,
         SERDES_MODE    => "SLAVE",
         TRAIN_PATTERN  => 0,
         OUTPUT_MODE    => "SINGLE_ENDED") --"DIFFERENTIAL") --
       port map (
        D1         => oserdes_pdat(0),
        D2         => oserdes_pdat(1),
        D3         => oserdes_pdat(2),
        D4         => oserdes_pdat(3),
        T1         => '0',
        T2         => '0',
        T3         => '0',
        T4         => '0',
        SHIFTIN1   => shiftout1_m, --ocascade_ms_d(pin_count),
        SHIFTIN2   => shiftout2_m, --ocascade_ms_t(pin_count),
        SHIFTIN3   => '1',
        SHIFTIN4   => '1',
        SHIFTOUT1  => open,
        SHIFTOUT2  => open,
        SHIFTOUT3  => shiftout3_s, --ocascade_sm_d(pin_count),
        SHIFTOUT4  => shiftout4_s, --ocascade_sm_t(pin_count),
        TRAIN      => '0',
        OCE        => '1',
        CLK0       => oserdes_ioclk,--pll_clkout0,  
        CLK1       => '0',
        CLKDIV     => clk,
        OQ         => open,
        TQ         => open,
        IOCE       => oserdes_stb,         
        TCE        => '1',
        RST        => reset); 

end arch_oserdes_01;