-------------------------------------------------------
-- Design Name : outer_pixel_delay_cal_01
-- File Name   : outer_pixel_delay_cal_01.vhd
-- Device      : Spartan 6
-- Function    : use the central pixel for reference and adjust the delay of the outer
--               6x5 pixels until you get a safe trigger
-- Coder       : K.-H. Sulanke, DESY, 2017-03-06
-------------------------------------------------------
-- 2017-02-28  first design

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all ;

library unisim ;
use unisim.vcomponents.all ;
use work.trigger_3nn_pkg.all;

entity outer_pixel_delay_cal_01 is
  generic (
           CLUSTER_NR     : std_logic_vector (2 downto 0) := B"000"
          );
  port (
        --clk               :  in std_logic ;   -- global clock input
        l0_del_sel_rg     :  in std_logic_vector (5 downto 0); -- bits 5..3= cluster_#, 2..0= pixel_#
        cl0_pdat3         :  in std_logic_vector (0 to 7);  -- cl0_pdat3 is the reference pixel
        clx_pdat0         :  in std_logic_vector (0 to 7);  -- all pulse shaped, according to the 
        clx_pdat1         :  in std_logic_vector (0 to 7);  -- TRIG_WIN register
        clx_pdat2         :  in std_logic_vector (0 to 7);  --
        clx_pdat3         :  in std_logic_vector (0 to 7);  --
        clx_pdat4         :  in std_logic_vector (0 to 7);  --
        trig_2of37        :  out std_logic_vector (0 to 7)  --
       );                                                 
  end outer_pixel_delay_cal_01 ;

architecture arch_outer_pixel_delay_cal_01 of outer_pixel_delay_cal_01 is
  
begin
 
 CHECK_COINCIDENCE: process (l0_del_sel_rg, cl0_pdat3, clx_pdat0, clx_pdat1, clx_pdat2, clx_pdat3, clx_pdat4)
    begin
     --if clk'event and clk = '1' then
      trig_2of37 <= X"00";                   -- for single pulses
      if l0_del_sel_rg(5 downto 3) = CLUSTER_NR then
        case pixel_remap(l0_del_sel_rg) is -- convert NectarCam pixel index to schematic / ucf index
          when 0 =>
            trig_2of37 <= clx_pdat0 and cl0_pdat3;          
          when 1 =>
            trig_2of37 <= clx_pdat1 and cl0_pdat3; 
          when 2 =>
            trig_2of37 <= clx_pdat2 and cl0_pdat3; 
          when 3 =>
            trig_2of37 <= clx_pdat3 and cl0_pdat3; 
          when 4 =>
            trig_2of37 <= clx_pdat4 and cl0_pdat3; 
          when others => 
            trig_2of37 <= X"00"; 
        end case;
      end if; -- l0_del_sel_rg(5 downto 3) = CLUSTER_NR       
     --end if; --clk'event and clk = '1'
   end process CHECK_COINCIDENCE; 

end arch_outer_pixel_delay_cal_01;