-------------------------------------------------------
-- Design Name : serdes_lh_edge_01
-- File Name   : serdes_lh_edge_01.vhd
-- Device      : Spartan 6
-- Function    : find a LH-edge in the serdes par. data
-- Coder       : K.-H. Sulanke, DESY, 2017-09-18
-------------------------------------------------------
--

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all ;

library unisim ;
use unisim.vcomponents.all ;

entity serdes_lh_edge_01 is
  port 	(
         clk            :  in  std_logic ;                 -- pdat clock
         serdes_pdat    :  in  std_logic_vector (0 to 7);  -- serdes pdat
         lh_edge        :  out std_logic                   -- pulse of one clk length
        );
  end serdes_lh_edge_01 ;

architecture arch_serdes_lh_edge_01 of serdes_lh_edge_01 is

  signal serdes_pdat_p7  : std_logic :='0';  -- previous bit 7
 
begin
  
 GET_LH_EDGE: process (clk) -- find max. one LH-edge per clock
    begin
     if rising_edge(clk) then
       serdes_pdat_p7 <= serdes_pdat(7);
       if    serdes_pdat_p7='0' and serdes_pdat(0)='1' then lh_edge <= '1';
       elsif serdes_pdat(0)='0' and serdes_pdat(1)='1' then lh_edge <= '1';
       elsif serdes_pdat(1)='0' and serdes_pdat(2)='1' then lh_edge <= '1';
       elsif serdes_pdat(2)='0' and serdes_pdat(3)='1' then lh_edge <= '1';
       elsif serdes_pdat(3)='0' and serdes_pdat(4)='1' then lh_edge <= '1';
       elsif serdes_pdat(4)='0' and serdes_pdat(5)='1' then lh_edge <= '1';
       elsif serdes_pdat(5)='0' and serdes_pdat(6)='1' then lh_edge <= '1';
       elsif serdes_pdat(6)='0' and serdes_pdat(7)='1' then lh_edge <= '1';
       else                                                 lh_edge <= '0';    
       end if;
      end if; --rising_edge(clk)
    end process GET_LH_EDGE; 
    
end arch_serdes_lh_edge_01;