-------------------------------------------------------
-- Design Name : trig_1_of_37_01
-- File Name   : trig_1_of_37_01.vhd
-- Device      : Spartan 6
-- Function    : check all 37 pixel for vaild signals
-- Coder       : K.-H. Sulanke, DESY, 2017-09-21
-------------------------------------------------------


library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all ;

library unisim ;
use unisim.vcomponents.all ;
use work.trigger_3nn_pkg.all;

entity trig_1_of_37_01 is
  port (
        pixel_sel_rg     :  in std_logic_vector (5 downto 0); -- bits 5..3= cluster_#, 2..0= pixel_#
        NBOR_IN           :  in std_logic_vector (1 to 6); -- low active signal with pullup, indicating a neighbor cluster is connected
        cl0_pdat0, cl0_pdat1, cl0_pdat2, cl0_pdat3, cl0_pdat4, cl0_pdat5, cl0_pdat6  :  in std_logic_vector (0 to 7); 
        cl1_pdat0, cl1_pdat1, cl1_pdat2, cl1_pdat3, cl1_pdat4  :  in std_logic_vector (0 to 7); -- all pulse shaped, according to the 
        cl2_pdat0, cl2_pdat1, cl2_pdat2, cl2_pdat3, cl2_pdat4  :  in std_logic_vector (0 to 7);
        cl3_pdat0, cl3_pdat1, cl3_pdat2, cl3_pdat3, cl3_pdat4  :  in std_logic_vector (0 to 7);
        cl4_pdat0, cl4_pdat1, cl4_pdat2, cl4_pdat3, cl4_pdat4  :  in std_logic_vector (0 to 7);
        cl5_pdat0, cl5_pdat1, cl5_pdat2, cl5_pdat3, cl5_pdat4  :  in std_logic_vector (0 to 7);
        cl6_pdat0, cl6_pdat1, cl6_pdat2, cl6_pdat3, cl6_pdat4  :  in std_logic_vector (0 to 7);
        trig_1of37        :  out std_logic_vector (0 to 7)  --
       );                                                 
  end trig_1_of_37_01 ;

architecture arch_trig_1_of_37_01 of trig_1_of_37_01 is
  
begin
 
 PDAT_MUX: process (pixel_sel_rg, NBOR_IN,
                    cl0_pdat0, cl0_pdat1, cl0_pdat2, cl0_pdat3, cl0_pdat4, cl0_pdat5, cl0_pdat6,
                    cl1_pdat0, cl1_pdat1, cl1_pdat2, cl1_pdat3, cl1_pdat4,
                    cl2_pdat0, cl2_pdat1, cl2_pdat2, cl2_pdat3, cl2_pdat4,
                    cl3_pdat0, cl3_pdat1, cl3_pdat2, cl3_pdat3, cl3_pdat4,
                    cl4_pdat0, cl4_pdat1, cl4_pdat2, cl4_pdat3, cl4_pdat4,
                    cl5_pdat0, cl5_pdat1, cl5_pdat2, cl5_pdat3, cl5_pdat4, 
                    cl6_pdat0, cl6_pdat1, cl6_pdat2, cl6_pdat3, cl6_pdat4)
    begin
      trig_1of37 <= X"00";
      if pixel_sel_rg(5 downto 3) = B"000" then
        case pixel_remap(pixel_sel_rg) is -- convert NectarCam pixel index to schematic / ucf index
          when 0 =>
            trig_1of37 <= cl0_pdat0;          
          when 1 =>
            trig_1of37 <= cl0_pdat1; 
          when 2 =>
            trig_1of37 <= cl0_pdat2; 
          when 3 =>
            trig_1of37 <= cl0_pdat3; 
          when 4 =>
            trig_1of37 <= cl0_pdat4;
          when 5 =>
            trig_1of37 <= cl0_pdat5;
          when 6 =>
            trig_1of37 <= cl0_pdat6;
          when others => 
            trig_1of37 <= X"00"; 
        end case;
       end if; 
      if pixel_sel_rg(5 downto 3) = B"001" and NBOR_IN(1) = '0' then
        case pixel_remap(pixel_sel_rg) is 
          when 0 =>
            trig_1of37 <= cl1_pdat0;          
          when 1 =>
            trig_1of37 <= cl1_pdat1; 
          when 2 =>
            trig_1of37 <= cl1_pdat2; 
          when 3 =>
            trig_1of37 <= cl1_pdat3; 
          when 4 =>
            trig_1of37 <= cl1_pdat4; 
          when others => 
            trig_1of37 <= X"00"; 
        end case;
       end if; 
      if pixel_sel_rg(5 downto 3) = B"010" and NBOR_IN(2) = '0' then
        case pixel_remap(pixel_sel_rg) is 
          when 0 =>
            trig_1of37 <= cl2_pdat0;          
          when 1 =>
            trig_1of37 <= cl2_pdat1; 
          when 2 =>
            trig_1of37 <= cl2_pdat2; 
          when 3 =>
            trig_1of37 <= cl2_pdat3; 
          when 4 =>
            trig_1of37 <= cl2_pdat4; 
          when others => 
            trig_1of37 <= X"00"; 
        end case;
       end if; 
      if pixel_sel_rg(5 downto 3) = B"011" and NBOR_IN(3) = '0' then
        case pixel_remap(pixel_sel_rg) is 
          when 0 =>
            trig_1of37 <= cl3_pdat0;          
          when 1 =>
            trig_1of37 <= cl3_pdat1; 
          when 2 =>
            trig_1of37 <= cl3_pdat2; 
          when 3 =>
            trig_1of37 <= cl3_pdat3; 
          when 4 =>
            trig_1of37 <= cl3_pdat4; 
          when others => 
            trig_1of37 <= X"00"; 
        end case;
       end if; 
      if pixel_sel_rg(5 downto 3) = B"100" and NBOR_IN(4) = '0' then
        case pixel_remap(pixel_sel_rg) is 
          when 0 =>
            trig_1of37 <= cl4_pdat0;          
          when 1 =>
            trig_1of37 <= cl4_pdat1; 
          when 2 =>
            trig_1of37 <= cl4_pdat2; 
          when 3 =>
            trig_1of37 <= cl4_pdat3; 
          when 4 =>
            trig_1of37 <= cl4_pdat4; 
          when others => 
            trig_1of37 <= X"00"; 
        end case;
       end if; 
      if pixel_sel_rg(5 downto 3) = B"101" and NBOR_IN(5) = '0' then
        case pixel_remap(pixel_sel_rg) is 
          when 0 =>
            trig_1of37 <= cl5_pdat0;          
          when 1 =>
            trig_1of37 <= cl5_pdat1; 
          when 2 =>
            trig_1of37 <= cl5_pdat2; 
          when 3 =>
            trig_1of37 <= cl5_pdat3; 
          when 4 =>
            trig_1of37 <= cl5_pdat4; 
          when others => 
            trig_1of37 <= X"00"; 
        end case;
       end if; 
      if pixel_sel_rg(5 downto 3) = B"110" and NBOR_IN(6) = '0' then
        case pixel_remap(pixel_sel_rg) is 
          when 0 =>
            trig_1of37 <= cl6_pdat0;          
          when 1 =>
            trig_1of37 <= cl6_pdat1; 
          when 2 =>
            trig_1of37 <= cl6_pdat2; 
          when 3 =>
            trig_1of37 <= cl6_pdat3; 
          when 4 =>
            trig_1of37 <= cl6_pdat4; 
          when others => 
            trig_1of37 <= X"00"; 
        end case;
       end if; 
   end process PDAT_MUX; 

end arch_trig_1_of_37_01;