-------------------------------------------------------
-- Design Name : trig_pulse_shaper_03
-- File Name   : trig_pulse_shaper_03.vhd
-- Device      : Spartan 6
-- Function    : modifying the oserdes output (progr. pulse length)
--               adding n pulse of clk length
-- Coder       : K.-H. Sulanke, DESY, 2017-10-16
-------------------------------------------------------
-- 2017-03-01  pulse_width 4 bit now
-- 2017-03-06  "elsif" introduced 
-- 2017-10-16  pulse_pdat_last 7 bit wide now to remove compiler warning

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all ;

library unisim ;
use unisim.vcomponents.all ;

entity trig_pulse_shaper_03 is
  port 	(
         clk             :  in  std_logic ;                    -- pdat clock
         pulse_width     :  in  std_logic_vector (3 downto 0); -- length in clock cycles
         dead_time       :  in  std_logic_vector (7 downto 0);  -- trigger inactive, amount of clks
         reset           :  in  std_logic;
         oserdes_pdat    :  in  std_logic_vector (0 to 7);     -- serdes pdat
         oserdes_ioclk   :  in  std_logic;
         oserdes_stb     :  in  std_logic;
         oserdes_out     :  out std_logic   -- serial oserdes output
        );
  end trig_pulse_shaper_03 ;

architecture arch_trig_pulse_shaper_03 of trig_pulse_shaper_03 is

  --signal oserdes_rst     : std_logic;
  signal oserdes_pdat_p7 : std_logic :='0';  -- previous bit 7
  signal pulse_pdat      : std_logic_vector (0 to 7) := X"00";
  signal pulse_pdat_last : std_logic_vector (0 to 6) := B"0000_000";
  
  signal dead_time_ct    : std_logic_vector (7 downto 0);
  
  type state_type is (PULSE_IDLE, PULSE_NEXT, PULSE_WAIT);
  signal state : state_type := PULSE_IDLE;
  
  signal shiftout3_s : std_logic;
  signal shiftout4_s : std_logic;
  signal shiftout1_m : std_logic;
  signal shiftout2_m : std_logic;
 
begin

  
PULSE_SHAPE: process (clk)
     variable ct : std_logic_vector (3 downto 0);
    begin
     if rising_edge(clk) then
      if reset = '1' then
        state       <= PULSE_IDLE;
        pulse_pdat  <= X"00";
      else  
        if pulse_width = X"0" then  -- no pulse shaping !
         pulse_pdat <= oserdes_pdat;
        else  -- start pulse shaping                  
        
         oserdes_pdat_p7 <= oserdes_pdat(7);
         
         case state is
         
          when PULSE_IDLE =>
          
            if    oserdes_pdat_p7='0' and oserdes_pdat(0)='1' then pulse_pdat <= X"ff"; pulse_pdat_last <= B"0000_000" ;
            elsif oserdes_pdat(0)='0' and oserdes_pdat(1)='1' then pulse_pdat <= X"7f"; pulse_pdat_last <= B"1000_000" ;
            elsif oserdes_pdat(1)='0' and oserdes_pdat(2)='1' then pulse_pdat <= X"3f"; pulse_pdat_last <= B"1100_000" ;
            elsif oserdes_pdat(2)='0' and oserdes_pdat(3)='1' then pulse_pdat <= X"1f"; pulse_pdat_last <= B"1110_000" ;
            elsif oserdes_pdat(3)='0' and oserdes_pdat(4)='1' then pulse_pdat <= X"0f"; pulse_pdat_last <= B"1111_000" ;
            elsif oserdes_pdat(4)='0' and oserdes_pdat(5)='1' then pulse_pdat <= X"07"; pulse_pdat_last <= B"1111_100" ;
            elsif oserdes_pdat(5)='0' and oserdes_pdat(6)='1' then pulse_pdat <= X"03"; pulse_pdat_last <= B"1111_110" ;
            elsif oserdes_pdat(6)='0' and oserdes_pdat(7)='1' then pulse_pdat <= X"01"; pulse_pdat_last <= B"1111_111" ;
            end if; 
          
            if oserdes_pdat /= X"00" then
             state       <= PULSE_NEXT;
             ct          := pulse_width;
            else
             pulse_pdat      <= X"00";
            end if; 
            
          when PULSE_NEXT => 
   
            if ct /= X"00" then 
             ct := ct - '1';           
             pulse_pdat   <= X"ff";
            else
             pulse_pdat   <= (pulse_pdat_last & '0');
             dead_time_ct <= dead_time;           
             state        <= PULSE_WAIT;
            end if; 
             
          when PULSE_WAIT =>  
             pulse_pdat   <= X"00";
             if dead_time_ct = X"00" then
               state      <= PULSE_IDLE;
             else
               dead_time_ct <=  dead_time_ct - '1';
             end if;              
             
         end case; -- state machine
        end if; --  if pulse_width = X"0"
       end if; -- if reset = '1' 
     end if; -- if rising_edge(clk)
    end process PULSE_SHAPE; 


   j13_87_trig_out_OSERDES_bank0_m: OSERDES2
       generic map (
         DATA_RATE_OQ   => "SDR",
         DATA_RATE_OT   => "SDR",
         TRAIN_PATTERN  => 0,
         DATA_WIDTH     => 8,
         SERDES_MODE    => "MASTER",
         OUTPUT_MODE    => "SINGLE_ENDED") --"DIFFERENTIAL")
       port map (
        D1         => pulse_pdat(4),
        D2         => pulse_pdat(5),
        D3         => pulse_pdat(6),
        D4         => pulse_pdat(7),
        T1         => '0',
        T2         => '0',
        T3         => '0',
        T4         => '0',
        SHIFTIN1   => '1',
        SHIFTIN2   => '1',
        SHIFTIN3   => shiftout3_s, --ocascade_sm_d(pin_count),
        SHIFTIN4   => shiftout4_s, --ocascade_sm_t(pin_count),
        SHIFTOUT1  => shiftout1_m, --ocascade_ms_d(pin_count),
        SHIFTOUT2  => shiftout2_m, --ocascade_ms_t(pin_count),
        SHIFTOUT3  => open,
        SHIFTOUT4  => open,
        TRAIN      => '0',
        OCE        => '1',--clock_enable,
        CLK0       => oserdes_ioclk,
        CLK1       => '0',
        CLKDIV     => clk,
        OQ         => oserdes_out,
        TQ         => open,
        IOCE       => oserdes_stb,
        TCE        => '1',
        RST        => reset);

   j13_87_trig_out_OSERDES_bank0_s : OSERDES2
       generic map (
         DATA_RATE_OQ   => "SDR",
         DATA_RATE_OT   => "SDR",
         DATA_WIDTH     => 8,
         SERDES_MODE    => "SLAVE",
         TRAIN_PATTERN  => 0,
         OUTPUT_MODE    => "SINGLE_ENDED") --"DIFFERENTIAL")
       port map (
        D1         => pulse_pdat(0),
        D2         => pulse_pdat(1),
        D3         => pulse_pdat(2),
        D4         => pulse_pdat(3),
        T1         => '0',
        T2         => '0',
        T3         => '0',
        T4         => '0',
        SHIFTIN1   => shiftout1_m, --ocascade_ms_d(pin_count),
        SHIFTIN2   => shiftout2_m, --ocascade_ms_t(pin_count),
        SHIFTIN3   => '1',
        SHIFTIN4   => '1',
        SHIFTOUT1  => open,
        SHIFTOUT2  => open,
        SHIFTOUT3  => shiftout3_s, --ocascade_sm_d(pin_count),
        SHIFTOUT4  => shiftout4_s, --ocascade_sm_t(pin_count),
        TRAIN      => '0',
        OCE        => '1',
        CLK0       => oserdes_ioclk,--pll_clkout0,  
        CLK1       => '0',
        CLKDIV     => clk,
        OQ         => open,
        TQ         => open,
        IOCE       => oserdes_stb,         
        TCE        => '1',
        RST        => reset); 

end arch_trig_pulse_shaper_03;