-- last change: 2017-12-20

library ieee;
    use ieee.std_logic_1164.all;
    use ieee.std_logic_unsigned.all;
    use ieee.numeric_std.all;

-- PACKAGE DECLARATION
package trigger_3nn_pkg is
-- FUNCTION DECLARATION

    function pix_trig_3_of_19_asyn_01 
              (a,b,c,d,e,f,g,e1,d1,e2,d2,e3,d3,e4,d4,e5,d5,e6,d6 : in std_logic;
               nbor_in : in std_logic_vector(1 to 6)
              )
               return std_logic;
 
    function pix_trig_3_of_19_asyn_02 
              (a,b,c,d,e,f,g,e1,d1,e2,d2,e3,d3,e4,d4,e5,d5,e6,d6 : in std_logic;
               nbor_in   : in std_logic_vector(1 to 6);
               trig_mskc : in std_logic_vector(6 downto 0);
               trig_mskn : in std_logic_vector(6 downto 1)               
              )
               return std_logic;

    function pix_trig_3_of_19_asyn_03 
              (a,b,c,d,e,f,g,e1,d1,e2,d2,e3,d3,e4,d4,e5,d5,e6,d6 : in std_logic;
               nbor_in   : in std_logic_vector(1 to 6);
               trig_mskc : in std_logic_vector(6 downto 0);
               trig_mskn : in std_logic_vector(6 downto 1)               
              )
               return std_logic;

    function pix_trig_3_of_19_asyn_04 
              (a,b,c,d,e,f,g,e1,d1,e2,d2,e3,d3,e4,d4,e5,d5,e6,d6 : in std_logic;
               nbor_in   : in std_logic_vector(1 to 6)               
              )
               return std_logic;
 
    function pix_trig_3_of_11_asyn_01 (a,c,d,e,f,h,i,j,k,l,m : in std_logic)
               return std_logic;
 
    function pix_trig_1_of_7_asyn_01 (a,b,c,d,e,f,g : in std_logic)
               return std_logic;
               
    function pix_trig_1_of_7_asyn_02 ( pix_0_to_6 : in std_logic_vector(0 to 6);
                                       trig_mskc  : in std_logic_vector(6 downto 0)) 
               return std_logic;

    function pix_trig_1_of_5_asyn_01 (a,b,c,d,e : in std_logic)
               return std_logic;
 
    function bcd4tobin12 (bcd4 : in std_logic_vector) return std_logic_vector;
 
    function bcd3tobin8 (bcd3 : in std_logic_vector) return std_logic_vector;

    --function pixel_remap_bak (pixel_pos : in std_logic_vector(5 downto 0)) return integer;

    function pixel_remap (pixel_pos : in std_logic_vector(5 downto 0)) return integer;

    function pixel_remap_cl0 (pixel_nr : in std_logic_vector(2 downto 0)) return integer;

    function pixel_remap_cl1to6(pixel_pos : in std_logic_vector(5 downto 0)) return integer;

    function pixel_remap_cl1 (pixel_nr : in integer range 0 to 4) return integer;
    function pixel_remap_cl2 (pixel_nr : in integer range 0 to 4) return integer;
    function pixel_remap_cl3 (pixel_nr : in integer range 0 to 4) return integer;
    function pixel_remap_cl4 (pixel_nr : in integer range 0 to 4) return integer;
    function pixel_remap_cl5 (pixel_nr : in integer range 0 to 4) return integer;
    function pixel_remap_cl6 (pixel_nr : in integer range 0 to 4) return integer;

    
 end trigger_3nn_pkg;


package body trigger_3nn_pkg is


-------------------------------------------------------
-- Function name   : pix_trig_3_of_19_asyn_01
-- Description     : Asynchronous 3NN trigger function for the center 7 + 12 
--                   surrounding pixels of a 37 pixels cluster
-- Project         : CTA, digital trigger board (dig_trigger_v2)
-- Coder           : K.-H. Sulanke, DESY
-- Date            : 2013-11-22
-------------------------------------------------------

function pix_trig_3_of_19_asyn_01 (a,b,c,d,e,f,g,e1,d1,e2,d2,e3,d3,e4,d4,e5,d5,e6,d6 : in std_logic;
                                   nbor_in : in std_logic_vector(1 to 6) ) 
  return std_logic is
   variable sub_trig	: std_logic_vector (0 to 6); 
   variable trig			: std_logic;
  begin
    sub_trig(0)	:=   (a AND b AND c)  -- inner 6(0) triangles
                  OR (a AND c AND d)	
                  OR (a AND d AND e)	
                  OR (a AND e AND f)	
                  OR (a AND f AND g)	
                  OR (a AND g AND b);
    
     sub_trig(1) :=  (b AND a AND d)		-- inner 120 triangles
                  OR (c AND a AND e)	
                  OR (d AND a AND f)	
                  OR (e AND a AND g)	
                  OR (f AND a AND b)	
                  OR (g AND a AND c)	;
        
     sub_trig(2)	:= (b AND c AND d)		-- outer 3 w/o center pixel trig_mskc(3)
                  OR (c AND d AND e)     
                  OR (d AND e AND f)	-- added 2017-02-09
                  OR (e AND f AND g)	
                  OR (f AND g AND b)	
                  OR (g AND b AND c) ;        
                          
     sub_trig(3)	:= (b AND a AND e)		-- inner diagonales
                  OR (c AND a AND f)	
                  OR (d AND a AND g)	;
                  
     sub_trig(4)	:= (a AND g AND d3 and not nbor_in(3)) -- including surrounding pixels from cluster 3,1,2
                  OR (a AND g AND e2 and not nbor_in(2)) -- nbor_in are low active inputs with pullup
                  OR (a AND g AND d2 and not nbor_in(2)) -- '1' means, neighbor cluster do not exist
                  OR (a AND b AND d2 and not nbor_in(2))
                  OR (a AND b AND e1 and not nbor_in(1))
                  OR (a AND b AND d1 and not nbor_in(1));

     sub_trig(5)	:= (a AND c AND d1 and not nbor_in(1)) -- including surrounding pixels from cluster 1,6,5
                  OR (a AND c AND e6 and not nbor_in(6))
                  OR (a AND c AND d6 and not nbor_in(6))
                  OR (a AND d AND d6 and not nbor_in(6))
                  OR (a AND d AND e5 and not nbor_in(5))
                  OR (a AND d AND d5 and not nbor_in(5));

     sub_trig(6)	:= (a AND e AND d5 and not nbor_in(5)) -- including surrounding pixels from cluster 5,4,3
                  OR (a AND e AND e4 and not nbor_in(4))
                  OR (a AND e AND d4 and not nbor_in(4))
                  OR (a AND f AND d4 and not nbor_in(4))
                  OR (a AND f AND e3 and not nbor_in(3))
                  OR (a AND f AND d3 and not nbor_in(3));
        
        
        trig		  := (sub_trig(0))
                  OR (sub_trig(1))
                  OR (sub_trig(2))
                  OR (sub_trig(3))
                  OR (sub_trig(4))
                  OR (sub_trig(5))
                  OR (sub_trig(6));
    
   return trig;   
  end function pix_trig_3_of_19_asyn_01;


-------------------------------------------------------
-- Function name   : pix_trig_3_of_19_asyn_02
-- Description     : Asynchronous 3NN trigger function for the center 7 + 12 
--                   surrounding pixels of a 37 pixels cluster
--                   adapted to NectarCam pixel numbering, with view on focal plane
-- Project         : CTA, digital trigger board (dig_trigger_v2..3)
-- Coder           : K.-H. Sulanke, DESY
-- Date            : 2016-09-29
-------------------------------------------------------

function pix_trig_3_of_19_asyn_02 (a,b,c,d,e,f,g,e1,d1,e2,d2,e3,d3,e4,d4,e5,d5,e6,d6 : in std_logic;
                                   nbor_in   : in std_logic_vector(1 to 6);
                                   trig_mskc : in std_logic_vector(6 downto 0);
                                   trig_mskn : in std_logic_vector(6 downto 1)                                   
                                   ) 
  return std_logic is
   variable sub_trig	: std_logic_vector (0 to 6); 
   variable trig			: std_logic;
  begin
    sub_trig(0)	:=   (a AND b AND c AND trig_mskc(3) AND trig_mskc(1) AND trig_mskc(0)) -- inner 6(0) triangles
                  OR (a AND c AND d AND trig_mskc(3) AND trig_mskc(0) AND trig_mskc(6))
                  OR (a AND d AND e AND trig_mskc(3) AND trig_mskc(6) AND trig_mskc(5))
                  OR (a AND e AND f AND trig_mskc(3) AND trig_mskc(5) AND trig_mskc(4))
                  OR (a AND f AND g AND trig_mskc(3) AND trig_mskc(4) AND trig_mskc(2))
                  OR (a AND g AND b AND trig_mskc(3) AND trig_mskc(2) AND trig_mskc(1))
                  ;                     
                                        
     sub_trig(1) :=  (b AND a AND d AND trig_mskc(1) AND trig_mskc(3) AND trig_mskc(6))-- inner 120 triangles
                  OR (c AND a AND e AND trig_mskc(0) AND trig_mskc(3) AND trig_mskc(5))
                  OR (d AND a AND f AND trig_mskc(6) AND trig_mskc(3) AND trig_mskc(4))
                  OR (e AND a AND g AND trig_mskc(5) AND trig_mskc(3) AND trig_mskc(2))
                  OR (f AND a AND b AND trig_mskc(4) AND trig_mskc(3) AND trig_mskc(1))
                  OR (g AND a AND c AND trig_mskc(2) AND trig_mskc(3) AND trig_mskc(0))
                  ;                     
                                        
     sub_trig(2)	:= (b AND c AND d AND trig_mskc(1) AND trig_mskc(0) AND trig_mskc(6))-- outer 3 w/o center pixel trig_mskc(3)                  OR (c AND d AND e)	
                  OR (d AND e AND f AND trig_mskc(6) AND trig_mskc(5) AND trig_mskc(4))
                  OR (c AND d AND e AND trig_mskc(0) AND trig_mskc(6) AND trig_mskc(5)) -- added 2017-02-09
                  OR (e AND f AND g AND trig_mskc(5) AND trig_mskc(4) AND trig_mskc(2))
                  OR (f AND g AND b AND trig_mskc(4) AND trig_mskc(2) AND trig_mskc(1))
                  OR (g AND b AND c AND trig_mskc(2) AND trig_mskc(1) AND trig_mskc(0))
                  ;                     
                                            
     sub_trig(3)	:= (b AND a AND e AND trig_mskc(1) AND trig_mskc(3) AND trig_mskc(5))-- inner diagonales
                  OR (c AND a AND f AND trig_mskc(0) AND trig_mskc(3) AND trig_mskc(4))
                  OR (d AND a AND g AND trig_mskc(6) AND trig_mskc(3) AND trig_mskc(2))
                  ;
                  
     sub_trig(4)	:= (a AND g AND d2 and trig_mskn(2) and not nbor_in(2)) -- including surrounding pixels from cluster 3,1,2
                  OR (a AND g AND e2 and trig_mskn(2) and not nbor_in(2)) -- nbor_in are low active inputs with pullup
                  OR (a AND g AND d1 and trig_mskn(1) and not nbor_in(1)) -- '1' means, neighbor cluster do not exist
                  OR (a AND b AND d1 and trig_mskn(1) and not nbor_in(1))
                  OR (a AND b AND e1 and trig_mskn(1) and not nbor_in(1))
                  OR (a AND b AND d6 and trig_mskn(6) and not nbor_in(6));

     sub_trig(5)	:= (a AND c AND d6 and trig_mskn(6) and not nbor_in(6)) -- including surrounding pixels from cluster 1,6,5
                  OR (a AND c AND e6 and trig_mskn(6) and not nbor_in(6))
                  OR (a AND c AND d5 and trig_mskn(5) and not nbor_in(5))
                  OR (a AND d AND d5 and trig_mskn(5) and not nbor_in(5))
                  OR (a AND d AND e5 and trig_mskn(5) and not nbor_in(5))
                  OR (a AND d AND d4 and trig_mskn(4) and not nbor_in(4));

     sub_trig(6)	:= (a AND e AND d4 and trig_mskn(4) and not nbor_in(4)) -- including surrounding pixels from cluster 5,4,3
                  OR (a AND e AND e4 and trig_mskn(4) and not nbor_in(4))
                  OR (a AND e AND d3 and trig_mskn(3) and not nbor_in(3))
                  OR (a AND f AND d3 and trig_mskn(3) and not nbor_in(3))
                  OR (a AND f AND e3 and trig_mskn(3) and not nbor_in(3))
                  OR (a AND f AND d2 and trig_mskn(2) and not nbor_in(2));
        
        
        trig		  := (sub_trig(0))
                  OR (sub_trig(1))
                  OR (sub_trig(2))
                  OR (sub_trig(3))
                  OR (sub_trig(4))
                  OR (sub_trig(5))
                  OR (sub_trig(6));
    
   return trig;   
  end function pix_trig_3_of_19_asyn_02;
  


-------------------------------------------------------
-- Function name   : pix_trig_3_of_19_asyn_03
-- Description     : Asynchronous 3NN trigger function for the center 7 + 12 
--                   surrounding pixels of a 37 pixels cluster
--                   adapted to NectarCam pixel numbering, with view on focal plane
--                   corrected for the MiniCam pixel / cluster mapping  
-- Project         : CTA, digital trigger board (dig_trigger_v2..3)
-- Coder           : K.-H. Sulanke, DESY
-- Date            : 2017-04-24
-------------------------------------------------------

function pix_trig_3_of_19_asyn_03 (a,b,c,d,e,f,g,e1,d1,e2,d2,e3,d3,e4,d4,e5,d5,e6,d6 : in std_logic;
                                   nbor_in   : in std_logic_vector(1 to 6);
                                   trig_mskc : in std_logic_vector(6 downto 0);
                                   trig_mskn : in std_logic_vector(6 downto 1)                                   
                                   ) 
  return std_logic is
   variable sub_trig	: std_logic_vector (0 to 6); 
   variable trig			: std_logic;
  begin
    sub_trig(0)	:=   (a AND b AND c AND trig_mskc(3) AND trig_mskc(1) AND trig_mskc(0)) -- inner 6(0) triangles
                  OR (a AND c AND d AND trig_mskc(3) AND trig_mskc(0) AND trig_mskc(6))
                  OR (a AND d AND e AND trig_mskc(3) AND trig_mskc(6) AND trig_mskc(5))
                  OR (a AND e AND f AND trig_mskc(3) AND trig_mskc(5) AND trig_mskc(4))
                  OR (a AND f AND g AND trig_mskc(3) AND trig_mskc(4) AND trig_mskc(2))
                  OR (a AND g AND b AND trig_mskc(3) AND trig_mskc(2) AND trig_mskc(1))
                  ;                     
                                        
     sub_trig(1) :=  (b AND a AND d AND trig_mskc(1) AND trig_mskc(3) AND trig_mskc(6))-- inner 120 triangles
                  OR (c AND a AND e AND trig_mskc(0) AND trig_mskc(3) AND trig_mskc(5))
                  OR (d AND a AND f AND trig_mskc(6) AND trig_mskc(3) AND trig_mskc(4))
                  OR (e AND a AND g AND trig_mskc(5) AND trig_mskc(3) AND trig_mskc(2))
                  OR (f AND a AND b AND trig_mskc(4) AND trig_mskc(3) AND trig_mskc(1))
                  OR (g AND a AND c AND trig_mskc(2) AND trig_mskc(3) AND trig_mskc(0))
                  ;                     
                                        
     sub_trig(2)	:= (b AND c AND d AND trig_mskc(1) AND trig_mskc(0) AND trig_mskc(6))-- outer 3 w/o center pixel trig_mskc(3)                  OR (c AND d AND e)	
                  OR (d AND e AND f AND trig_mskc(6) AND trig_mskc(5) AND trig_mskc(4))
                  OR (c AND d AND e AND trig_mskc(0) AND trig_mskc(6) AND trig_mskc(5)) -- added 2017-02-09
                  OR (e AND f AND g AND trig_mskc(5) AND trig_mskc(4) AND trig_mskc(2))
                  OR (f AND g AND b AND trig_mskc(4) AND trig_mskc(2) AND trig_mskc(1))
                  OR (g AND b AND c AND trig_mskc(2) AND trig_mskc(1) AND trig_mskc(0))
                  ;                     
                                            
     sub_trig(3)	:= (b AND a AND e AND trig_mskc(1) AND trig_mskc(3) AND trig_mskc(5))-- inner diagonales
                  OR (c AND a AND f AND trig_mskc(0) AND trig_mskc(3) AND trig_mskc(4))
                  OR (d AND a AND g AND trig_mskc(6) AND trig_mskc(3) AND trig_mskc(2))
                  ;
                  
     sub_trig(4)	:= (a AND g AND d4 and trig_mskn(4) and not nbor_in(4)) -- including surrounding pixels from cluster 3,1,2
                  OR (a AND g AND e4 and trig_mskn(4) and not nbor_in(4)) -- nbor_in are low active inputs with pullup
                  OR (a AND g AND d3 and trig_mskn(3) and not nbor_in(3)) -- '1' means, neighbor cluster do not exist
                  OR (a AND b AND d3 and trig_mskn(3) and not nbor_in(3))
                  OR (a AND b AND e3 and trig_mskn(3) and not nbor_in(3))
                  OR (a AND b AND d2 and trig_mskn(2) and not nbor_in(2));

     sub_trig(5)	:= (a AND c AND d2 and trig_mskn(2) and not nbor_in(2)) -- including surrounding pixels from cluster 1,6,5
                  OR (a AND c AND e2 and trig_mskn(2) and not nbor_in(2))
                  OR (a AND c AND d1 and trig_mskn(1) and not nbor_in(1))
                  OR (a AND d AND d1 and trig_mskn(1) and not nbor_in(1))
                  OR (a AND d AND e1 and trig_mskn(1) and not nbor_in(1))
                  OR (a AND d AND d6 and trig_mskn(6) and not nbor_in(6));

     sub_trig(6)	:= (a AND e AND d6 and trig_mskn(6) and not nbor_in(6)) -- including surrounding pixels from cluster 5,4,3
                  OR (a AND e AND e6 and trig_mskn(6) and not nbor_in(6))
                  OR (a AND e AND d5 and trig_mskn(5) and not nbor_in(5))
                  OR (a AND f AND d5 and trig_mskn(5) and not nbor_in(5))
                  OR (a AND f AND e5 and trig_mskn(5) and not nbor_in(5))
                  OR (a AND f AND d4 and trig_mskn(4) and not nbor_in(4));
        
        
        trig		  := (sub_trig(0))
                  OR (sub_trig(1))
                  OR (sub_trig(2))
                  OR (sub_trig(3))
                  OR (sub_trig(4))
                  OR (sub_trig(5))
                  OR (sub_trig(6));
    
   return trig;   
  end function pix_trig_3_of_19_asyn_03;
  

-------------------------------------------------------
-- Function name   : pix_trig_3_of_19_asyn_04
-- Description     : Asynchronous 3NN trigger function for the center 7 + 12 
--                   surrounding pixels of a 37 pixels cluster
--                   adapted to NectarCam pixel numbering, with view on focal plane
--                   corrected for the MiniCam pixel / cluster mapping 
--                   w/o trig_mskc, trig_mskn now 
-- Project         : CTA, digital trigger board (dig_trigger_v2..3)
-- Coder           : K.-H. Sulanke, DESY
-- Date            : 2017-12-20
-------------------------------------------------------

function pix_trig_3_of_19_asyn_04 (a,b,c,d,e,f,g,e1,d1,e2,d2,e3,d3,e4,d4,e5,d5,e6,d6 : in std_logic;
                                   nbor_in   : in std_logic_vector(1 to 6)                                   
                                   ) 
  return std_logic is
   variable sub_trig	: std_logic_vector (0 to 6); 
   variable trig			: std_logic;
  begin
    sub_trig(0)	:=   (a AND b AND c ) -- inner 6(0) triangles
                  OR (a AND c AND d )
                  OR (a AND d AND e )
                  OR (a AND e AND f )
                  OR (a AND f AND g )
                  OR (a AND g AND b )
                  ;                 
                                    
     sub_trig(1) :=  (b AND a AND d )-- inner 120 triangles
                  OR (c AND a AND e )
                  OR (d AND a AND f )
                  OR (e AND a AND g )
                  OR (f AND a AND b )
                  OR (g AND a AND c )
                  ;                 
                                    
     sub_trig(2)	:= (b AND c AND d )-- outer 3 w/o center pixel trig_mskc(3)                  OR (c AND d AND e)	
                  OR (d AND e AND f )
                  OR (c AND d AND e ) -- added 2017-02-09
                  OR (e AND f AND g )
                  OR (f AND g AND b )
                  OR (g AND b AND c )
                  ;                     
                                            
     sub_trig(3)	:= (b AND a AND  e )-- inner diagonales
                  OR (c AND a AND  f )
                  OR (d AND a AND  g )
                  ;
                  
     sub_trig(4)	:= (a AND g AND d4 and not nbor_in(4)) -- including surrounding pixels from cluster 3,1,2
                  OR (a AND g AND e4 and not nbor_in(4)) -- nbor_in are low active inputs with pullup
                  OR (a AND g AND d3 and not nbor_in(3)) -- '1' means, neighbor cluster do not exist
                  OR (a AND b AND d3 and not nbor_in(3))
                  OR (a AND b AND e3 and not nbor_in(3))
                  OR (a AND b AND d2 and not nbor_in(2));

     sub_trig(5)	:= (a AND c AND d2 and not nbor_in(2)) -- including surrounding pixels from cluster 1,6,5
                  OR (a AND c AND e2 and not nbor_in(2))
                  OR (a AND c AND d1 and not nbor_in(1))
                  OR (a AND d AND d1 and not nbor_in(1))
                  OR (a AND d AND e1 and not nbor_in(1))
                  OR (a AND d AND d6 and not nbor_in(6));

     sub_trig(6)	:= (a AND e AND d6 and not nbor_in(6)) -- including surrounding pixels from cluster 5,4,3
                  OR (a AND e AND e6 and not nbor_in(6))
                  OR (a AND e AND d5 and not nbor_in(5))
                  OR (a AND f AND d5 and not nbor_in(5))
                  OR (a AND f AND e5 and not nbor_in(5))
                  OR (a AND f AND d4 and not nbor_in(4));
        
        
        trig		  := (sub_trig(0))
                  OR (sub_trig(1))
                  OR (sub_trig(2))
                  OR (sub_trig(3))
                  OR (sub_trig(4))
                  OR (sub_trig(5))
                  OR (sub_trig(6));
    
   return trig;   
  end function pix_trig_3_of_19_asyn_04;
  

  
-------------------------------------------------------
-- Function name   : pix_trig_3_of_11_asyn_01
-- Description     : Asynchronous 3NN trigger function for the outer 11 pixels of a 37 pixels cluster 
-- Project         : CTA, digital trigger board (dig_trigger_v2)
-- Coder           : K.-H. Sulanke, DESY
-- Date            : 2013-12-02
-------------------------------------------------------

function pix_trig_3_of_11_asyn_01 (a,c,d,e,f,h,i,j,k,l,m : in std_logic)
  return std_logic is
   variable sub_trig	  : std_logic_vector (0 to 4); 
   variable trig				: std_logic;
  begin

	sub_trig(0)	:=               -- a,c,d,e,f,k
						   (a AND c AND d)
						OR (a AND c AND e)               
						OR (a AND c AND f)            
						OR (a AND d AND f)
						OR (a AND d AND e)
 						OR (a AND d AND k)
						OR (a AND e AND f)            
						OR (a AND e AND k)
           	OR (c AND d AND e)	
						OR (d AND e AND f)
            ;
	
	sub_trig(1)	:= 	          -- a,c,d,h,i,j
						   (h AND c AND a)	
						OR (h AND c AND d)	
						OR (h AND i AND c)	
						OR (h AND i AND d)	
						OR (h AND i AND j)
						OR (i AND c AND a)	
						OR (i AND c AND d)	
						OR (i AND d AND a)
						OR (j AND d AND a)
            ;
						
	sub_trig(2)	:= 
               (i AND d AND e) -- c,d,e,i,j,k
						OR (i AND d AND j) 
						OR (i AND d AND k) 
						OR (i AND j AND k) 
            OR (j AND i AND c)
						OR (j AND d AND c)
            OR (j AND d AND e) 
						OR (j AND k AND d) 
						OR (j AND k AND e)
            OR (k AND d AND c)
            OR (k AND d AND e)
            ;

						
	sub_trig(3)	:=             -- a,e,f,k,l,m
               (k AND e AND f) 
						OR (k AND e AND m) 
						OR (k AND e AND l) 
						OR (k AND l AND m)
						OR (l AND e AND a) 
						OR (l AND e AND f) 
						OR (l AND m AND e) 
						OR (l AND m AND f)
						OR (m AND e AND a) 
						OR (m AND e AND f) 
						OR (m AND f AND a)
            ;
	
	sub_trig(4)	:= 
               (m AND e AND d) 	-- e,d,j,k,l,m           
            OR (l AND e AND d)                       
						OR (j AND k AND l) 
            OR (k AND l AND d)
            ;
	                           
	trig				:= 
               (sub_trig(0))
						OR (sub_trig(1))
						OR (sub_trig(2))
						OR (sub_trig(3))
						OR (sub_trig(4))
            ;
            
   return trig;
   
  end function pix_trig_3_of_11_asyn_01;

-------------------------------------------------------
-- Function name   : pix_trig_1_of_7_asyn_01
-- Description     : Asynchronous OR-trigger function for 
-- a 7 pixel cluster 
-- Project         : CTA, digital trigger board (dig_trigger_v2)
-- Coder           : K.-H. Sulanke, DESY
-- Date            : 2015-09-08
-------------------------------------------------------

 function pix_trig_1_of_7_asyn_01 (a,b,c,d,e,f,g : in std_logic)
  return std_logic is
   variable sub_trig	: std_logic_vector (0 to 1); 
   variable trig			: std_logic;
  begin
   sub_trig(0)  := a OR b OR c OR d;
   sub_trig(1)  := e OR f OR g;
   
	 trig	        := (sub_trig(0)) OR (sub_trig(1));
   
   return trig;
  end function pix_trig_1_of_7_asyn_01;



-------------------------------------------------------
-- Function name   : pix_trig_1_of_7_asyn_02
-- Description     : exclusive asynchronous OR-trigger function for 
--                   a 7 pixel cluster 
-- Project         : CTA, digital trigger board (dig_trigger_v2)
-- Coder           : K.-H. Sulanke, DESY
-- Date            : 2015-09-08
-------------------------------------------------------

 function pix_trig_1_of_7_asyn_02 ( pix_0_to_6 : in std_logic_vector(0 to 6);
                                    trig_mskc  : in std_logic_vector(6 downto 0)) 
  return std_logic is
   --variable pix_0_to_6 : std_logic_vector(0 to 6);
   variable trig			 : std_logic;
  begin
--    pix_0_to_6 := (a,b,c,d,e,f,g);
--    
--	  if ((pix_0_to_6 = B"1000000" and trig_mskc(0)='1') or
--        (pix_0_to_6 = B"0100000" and trig_mskc(1)='1') or
--        (pix_0_to_6 = B"0010000" and trig_mskc(2)='1') or
--        (pix_0_to_6 = B"0001000" and trig_mskc(3)='1') or
--        (pix_0_to_6 = B"0000100" and trig_mskc(4)='1') or
--        (pix_0_to_6 = B"0000010" and trig_mskc(5)='1') or
--        (pix_0_to_6 = B"0000001" and trig_mskc(6)='1') ) then
--      trig := '1';
--    else
--      trig := '0';    
--    end if;
 	  if ((trig_mskc(0)='1' and pix_0_to_6(0)='1') or
        (trig_mskc(1)='1' and pix_0_to_6(1)='1') or
        (trig_mskc(2)='1' and pix_0_to_6(2)='1') or
        (trig_mskc(3)='1' and pix_0_to_6(3)='1') or
        (trig_mskc(4)='1' and pix_0_to_6(4)='1') or
        (trig_mskc(5)='1' and pix_0_to_6(5)='1') or
        (trig_mskc(6)='1' and pix_0_to_6(6)='1')) then
      trig := '1';
    else
      trig := '0';    
    end if;
   return trig;
  end function pix_trig_1_of_7_asyn_02;  

-------------------------------------------------------
-- Function name   : pix_trig_1_of_5_asyn_01
-- Description     : Asynchronous OR-trigger function for 
-- a 7 pixel cluster 
-- Project         : CTA, digital trigger board (dig_trigger_v2)
-- Coder           : K.-H. Sulanke, DESY
-- Date            : 2015-09-08
-------------------------------------------------------

 function pix_trig_1_of_5_asyn_01 (a,b,c,d,e : in std_logic)
                                    
  return std_logic is
    variable trig			: std_logic;
  begin
 	 trig	        := a OR b OR c OR d OR e;
  return trig;
 end function pix_trig_1_of_5_asyn_01;

-------------------------------------------------------
-- Function name   : bcd4tobin12
-- Description     : converts a 4 x 4 bit bcd pattern, made 
--                   from an ascii (uart) input to a 12 
--                   bit binary number
-- Coder           : K.-H. Sulanke, DESY
-------------------------------------------------------

function bcd4tobin12 (bcd4: in std_logic_vector) return std_logic_vector is
  variable bcd_nr    : std_logic_vector (15 downto 0);
  variable int_nr    : integer;
  variable bin_nr    : std_logic_vector (11 downto 0) := X"000";
begin
  
    bcd_nr := bcd4;
    int_nr := to_integer ( unsigned(bcd_nr ( 3 downto   0))) --*   1
           +  to_integer ( unsigned(bcd_nr ( 7 downto   4))) *    10
           +  to_integer ( unsigned(bcd_nr (11 downto   8))) *   100
           +  to_integer ( unsigned(bcd_nr (15 downto  12))) *  1000;
  
--  bin_nr := std_logic_vector( to_signed(to_integer (unsigned(bcd_nr (3 downto  0))) *    1, 12))
 --         + std_logic_vector( to_signed(to_integer (unsigned(bcd_nr (7 downto  4))) *   10, 12))
   --       + std_logic_vector( to_signed(to_integer (unsigned(bcd_nr(11 downto  8))) *  100, 12))
     --     + std_logic_vector( to_signed(to_integer (unsigned(bcd_nr(15 downto 12))) * 1000, 12));
   
  bin_nr := std_logic_vector( to_signed(int_nr, 12));
            
  return bin_nr;   
end function bcd4tobin12;


-------------------------------------------------------
-- Function name   : bcd3tobin8
-- Description     : converts a 3 x 4 bit bcd pattern, made 
--                   from an ascii (uart) input to an 8  
--                   bit binary number
-- Coder           : K.-H. Sulanke, DESY
-------------------------------------------------------

function bcd3tobin8 (bcd3: in std_logic_vector) return std_logic_vector is
  variable bcd_nr    : std_logic_vector (11 downto 0);
  variable int_nr    : integer;
  variable bin_nr    : std_logic_vector (7 downto 0) := X"00";
begin
  
    bcd_nr := bcd3;
    int_nr := to_integer ( unsigned(bcd_nr ( 3 downto   0))) --*   1
           +  to_integer ( unsigned(bcd_nr ( 7 downto   4))) *    10
           +  to_integer ( unsigned(bcd_nr (11 downto   8))) *   100;
  
    bin_nr := std_logic_vector( to_signed(int_nr, 8));          
  return bin_nr;
end function bcd3tobin8;

---------------------------------------------------------
---- Function name   : pixel_remap_bak
---- Description     : to translate from the NectarCam index schema to the schematic / .ucf index,
----                   converts the pixel position from 6 bit pattern 
----                   to an integer number, used as index
----                   needed for L0 delay pixel addressing (PIXEL_SEL reg.)
----                   correct ???, renamed to .._bak, 2017-09-21 
---- Coder           : K.-H. Sulanke, DESY,2016-10-05
---------------------------------------------------------
--
--function pixel_remap_bak (pixel_pos : in std_logic_vector(5 downto 0)) return integer is
--  variable cluster_nr  : std_logic_vector (2 downto 0);
--  variable pixel_nr    : std_logic_vector (2 downto 0);
--  variable pixel_nr_rmpd : integer;
--begin
--  cluster_nr := pixel_pos(5 downto 3);
--  pixel_nr   := pixel_pos(2 downto 0);
--  case cluster_nr is
--   when B"000" => 
--    case pixel_nr is
--     when B"000" => pixel_nr_rmpd := 0;
--     when B"001" => pixel_nr_rmpd := 1;
--     when B"010" => pixel_nr_rmpd := 2;
--     when B"011" => pixel_nr_rmpd := 3;
--     when B"100" => pixel_nr_rmpd := 4;
--     when B"101" => pixel_nr_rmpd := 5;
--     when B"110" => pixel_nr_rmpd := 6;
--     when others => NULL;
--    end case; 
--   when B"001" => 
--    case pixel_nr is
--     when B"000" => pixel_nr_rmpd := 4;
--     when B"011" => pixel_nr_rmpd := 3;
--     when B"100" => pixel_nr_rmpd := 0;
--     when B"101" => pixel_nr_rmpd := 1;
--     when B"110" => pixel_nr_rmpd := 2;
--     when others => NULL;
--    end case; 
--   when B"010" => 
--    case pixel_nr is
--     when B"000" => pixel_nr_rmpd := 2;
--     when B"001" => pixel_nr_rmpd := 4;
--     when B"011" => pixel_nr_rmpd := 3;
--     when B"101" => pixel_nr_rmpd := 0;
--     when B"110" => pixel_nr_rmpd := 1;
--     when others => NULL;
--    end case; 
--   when B"011" => 
--    case pixel_nr is
--     when B"000" => pixel_nr_rmpd := 1;
--     when B"001" => pixel_nr_rmpd := 2;
--     when B"010" => pixel_nr_rmpd := 4;
--     when B"011" => pixel_nr_rmpd := 3;
--     when B"110" => pixel_nr_rmpd := 0;
--     when others => NULL;
--    end case; 
--   when B"100" => 
--    case pixel_nr is
--     when B"000" => pixel_nr_rmpd := 0;
--     when B"001" => pixel_nr_rmpd := 1;
--     when B"010" => pixel_nr_rmpd := 2;
--     when B"011" => pixel_nr_rmpd := 3;
--     when B"100" => pixel_nr_rmpd := 4;
--     when others => NULL;
--    end case; 
--    when B"101" => 
--    case pixel_nr is
--     when B"001" => pixel_nr_rmpd := 0;
--     when B"010" => pixel_nr_rmpd := 1;
--     when B"011" => pixel_nr_rmpd := 3;
--     when B"100" => pixel_nr_rmpd := 2;
--     when B"101" => pixel_nr_rmpd := 4;
--     when others => NULL;
--    end case; 
--   when B"110" => 
--    case pixel_nr is
--     when B"010" => pixel_nr_rmpd := 0;
--     when B"011" => pixel_nr_rmpd := 3;
--     when B"100" => pixel_nr_rmpd := 1;
--     when B"101" => pixel_nr_rmpd := 2;
--     when B"110" => pixel_nr_rmpd := 4;
--     when others => NULL;
--    end case;
--   when others => NULL;    
--  end case; 
-- return pixel_nr_rmpd;
--end function pixel_remap_bak;
--    

-------------------------------------------------------
-- Function name   : pixel_remap
-- Description     : to translate from the NectarCam index schema to the schematic / .ucf index,
--                   converts the pixel position from 6 bit pattern 
--                   to an integer number, used as index
--                   needed for L0 delay pixel addressing (PIXEL_SEL reg.), corrected now  
-- Coder           : K.-H. Sulanke, DESY,2016-09-21
-------------------------------------------------------

function pixel_remap (pixel_pos : in std_logic_vector(5 downto 0)) return integer is
  variable cluster_nr  : std_logic_vector (2 downto 0);
  variable pixel_nr    : std_logic_vector (2 downto 0);
  variable pixel_nr_rmpd : integer;
begin
  cluster_nr := pixel_pos(5 downto 3);
  pixel_nr   := pixel_pos(2 downto 0);
  case cluster_nr is
   when B"000" => 
    case pixel_nr is
     when B"000" => pixel_nr_rmpd := 0;
     when B"001" => pixel_nr_rmpd := 1;
     when B"010" => pixel_nr_rmpd := 2;
     when B"011" => pixel_nr_rmpd := 3;
     when B"100" => pixel_nr_rmpd := 4;
     when B"101" => pixel_nr_rmpd := 5;
     when B"110" => pixel_nr_rmpd := 6;
     when others => NULL;
    end case;
   when B"001" => 
    case pixel_nr is
     when B"001" => pixel_nr_rmpd := 0;
     when B"010" => pixel_nr_rmpd := 1;
     when B"011" => pixel_nr_rmpd := 3;
     when B"100" => pixel_nr_rmpd := 2;
     when B"101" => pixel_nr_rmpd := 4;
     when others => NULL;
    end case; 
   when B"010" => 
    case pixel_nr is
     when B"010" => pixel_nr_rmpd := 0;
     when B"011" => pixel_nr_rmpd := 3;
     when B"100" => pixel_nr_rmpd := 1;
     when B"101" => pixel_nr_rmpd := 2;
     when B"110" => pixel_nr_rmpd := 4;
     when others => NULL;
    end case;    
   when B"011" => 
    case pixel_nr is
     when B"000" => pixel_nr_rmpd := 4;
     when B"011" => pixel_nr_rmpd := 3;
     when B"100" => pixel_nr_rmpd := 0;
     when B"101" => pixel_nr_rmpd := 1;
     when B"110" => pixel_nr_rmpd := 2;
     when others => NULL;
    end case; 
   when B"100" => 
    case pixel_nr is
     when B"000" => pixel_nr_rmpd := 2;
     when B"001" => pixel_nr_rmpd := 4;
     when B"011" => pixel_nr_rmpd := 3;
     when B"101" => pixel_nr_rmpd := 0;
     when B"110" => pixel_nr_rmpd := 1;
     when others => NULL;
    end case; 
   when B"101" => 
    case pixel_nr is
     when B"000" => pixel_nr_rmpd := 1;
     when B"001" => pixel_nr_rmpd := 2;
     when B"010" => pixel_nr_rmpd := 4;
     when B"011" => pixel_nr_rmpd := 3;
     when B"110" => pixel_nr_rmpd := 0;
     when others => NULL;
    end case; 
   when B"110" => 
    case pixel_nr is
     when B"000" => pixel_nr_rmpd := 0;
     when B"001" => pixel_nr_rmpd := 1;
     when B"010" => pixel_nr_rmpd := 2;
     when B"011" => pixel_nr_rmpd := 3;
     when B"100" => pixel_nr_rmpd := 4;
     when others => NULL;
    end case; 
   when others => NULL;    
  end case; 
 return pixel_nr_rmpd;
end function pixel_remap;  
 
 

-------------------------------------------------------
-- Function name   : pixel_remap_cl0
-- Description     : to translate from the NectarCam index schema to the schematic / .ucf index,
--                   converts the pixel position from 6 bit pattern 
--                   to an integer number, used as index
--                   needed for L0 delay pixel addressing (PIXEL_SEL reg.), corrected now  
-- Coder           : K.-H. Sulanke, DESY,2017-10-16
-------------------------------------------------------

function pixel_remap_cl0 (pixel_nr : in std_logic_vector(2 downto 0)) return integer is
  variable pixel_nr_rmpd : integer;
begin
    case pixel_nr is
     when B"000" => pixel_nr_rmpd := 0;
     when B"001" => pixel_nr_rmpd := 1;
     when B"010" => pixel_nr_rmpd := 2;
     when B"011" => pixel_nr_rmpd := 3;
     when B"100" => pixel_nr_rmpd := 4;
     when B"101" => pixel_nr_rmpd := 5;
     when B"110" => pixel_nr_rmpd := 6;
     when others => NULL;
    end case;
 return pixel_nr_rmpd;
end function pixel_remap_cl0;   
 
-------------------------------------------------------
-- Function name   : pixel_remap_cl1 .. cl6
-- Description     : to translate from the schematic / .ucf index (!!) to the NectarCam index schema,
--                   needed for pixel masking 
-- Coder           : K.-H. Sulanke, DESY,2017-12-20
-------------------------------------------------------

function pixel_remap_cl1 (pixel_nr : in integer range 0 to 4) return integer is
  variable pixel_nr_rmpd : integer;
begin
    case pixel_nr is
     when 0 => pixel_nr_rmpd := 1;
     when 1 => pixel_nr_rmpd := 2;
     when 3 => pixel_nr_rmpd := 3;
     when 2 => pixel_nr_rmpd := 4;
     when 4 => pixel_nr_rmpd := 5;
     when others => NULL;
    end case;
 return pixel_nr_rmpd;
end function pixel_remap_cl1;   
 
function pixel_remap_cl2 (pixel_nr : in integer range 0 to 4) return integer is
  variable pixel_nr_rmpd : integer;
begin
    case pixel_nr is
     when 0 => pixel_nr_rmpd := 2;
     when 3 => pixel_nr_rmpd := 3;
     when 1 => pixel_nr_rmpd := 4;
     when 2 => pixel_nr_rmpd := 5;
     when 4 => pixel_nr_rmpd := 6;
     when others => NULL;
    end case;
 return pixel_nr_rmpd;
end function pixel_remap_cl2;

function pixel_remap_cl3 (pixel_nr : in integer range 0 to 4) return integer is
  variable pixel_nr_rmpd : integer;
begin
    case pixel_nr is
     when 4 => pixel_nr_rmpd := 0;
     when 3 => pixel_nr_rmpd := 3;
     when 0 => pixel_nr_rmpd := 4;
     when 1 => pixel_nr_rmpd := 5;
     when 2 => pixel_nr_rmpd := 6;
     when others => NULL;
    end case;
 return pixel_nr_rmpd;
end function pixel_remap_cl3;

function pixel_remap_cl4 (pixel_nr : in integer range 0 to 4) return integer is
  variable pixel_nr_rmpd : integer;
begin
    case pixel_nr is
     when 2 => pixel_nr_rmpd := 0;
     when 4 => pixel_nr_rmpd := 1;
     when 3 => pixel_nr_rmpd := 3;
     when 0 => pixel_nr_rmpd := 5;
     when 1 => pixel_nr_rmpd := 6;
     when others => NULL;
    end case;
 return pixel_nr_rmpd;
end function pixel_remap_cl4;

function pixel_remap_cl5 (pixel_nr : in integer range 0 to 4) return integer is
  variable pixel_nr_rmpd : integer;
begin
    case pixel_nr is
     when 1 => pixel_nr_rmpd := 0;
     when 2 => pixel_nr_rmpd := 1;
     when 4 => pixel_nr_rmpd := 2;
     when 3 => pixel_nr_rmpd := 3;
     when 0 => pixel_nr_rmpd := 6;
     when others => NULL;
    end case;
 return pixel_nr_rmpd;
end function pixel_remap_cl5;

function pixel_remap_cl6 (pixel_nr : in integer range 0 to 4) return integer is
  variable pixel_nr_rmpd : integer;
begin
    case pixel_nr is
     when 0 => pixel_nr_rmpd := 0;
     when 1 => pixel_nr_rmpd := 1;
     when 2 => pixel_nr_rmpd := 2;
     when 3 => pixel_nr_rmpd := 3;
     when 4 => pixel_nr_rmpd := 4;
     when others => NULL;
    end case;
 return pixel_nr_rmpd;
end function pixel_remap_cl6;

  

-------------------------------------------------------
-- Function name   : pixel_remap_cl1to6
-- Description     : to translate from the NectarCam index schema to the schematic / .ucf index,
--                   converts the pixel position from 6 bit pattern 
--                   to an integer number, used as index
--                   needed for L0 delay pixel addressing (PIXEL_SEL reg.), corrected now  
-- Coder           : K.-H. Sulanke, DESY,2017-10-16
-------------------------------------------------------

function pixel_remap_cl1to6 (pixel_pos : in std_logic_vector(5 downto 0)) return integer is
  variable cluster_nr  : std_logic_vector (2 downto 0);
  variable pixel_nr    : std_logic_vector (2 downto 0);
  variable pixel_nr_rmpd : integer;
begin
  cluster_nr := pixel_pos(5 downto 3);
  pixel_nr   := pixel_pos(2 downto 0);
  case cluster_nr is
   when B"001" => 
    case pixel_nr is
     when B"001" => pixel_nr_rmpd := 0;
     when B"010" => pixel_nr_rmpd := 1;
     when B"011" => pixel_nr_rmpd := 3;
     when B"100" => pixel_nr_rmpd := 2;
     when B"101" => pixel_nr_rmpd := 4;
     when others => NULL;
    end case; 
   when B"010" => 
    case pixel_nr is
     when B"010" => pixel_nr_rmpd := 0;
     when B"011" => pixel_nr_rmpd := 3;
     when B"100" => pixel_nr_rmpd := 1;
     when B"101" => pixel_nr_rmpd := 2;
     when B"110" => pixel_nr_rmpd := 4;
     when others => NULL;
    end case;    
   when B"011" => 
    case pixel_nr is
     when B"000" => pixel_nr_rmpd := 4;
     when B"011" => pixel_nr_rmpd := 3;
     when B"100" => pixel_nr_rmpd := 0;
     when B"101" => pixel_nr_rmpd := 1;
     when B"110" => pixel_nr_rmpd := 2;
     when others => NULL;
    end case; 
   when B"100" => 
    case pixel_nr is
     when B"000" => pixel_nr_rmpd := 2;
     when B"001" => pixel_nr_rmpd := 4;
     when B"011" => pixel_nr_rmpd := 3;
     when B"101" => pixel_nr_rmpd := 0;
     when B"110" => pixel_nr_rmpd := 1;
     when others => NULL;
    end case; 
   when B"101" => 
    case pixel_nr is
     when B"000" => pixel_nr_rmpd := 1;
     when B"001" => pixel_nr_rmpd := 2;
     when B"010" => pixel_nr_rmpd := 4;
     when B"011" => pixel_nr_rmpd := 3;
     when B"110" => pixel_nr_rmpd := 0;
     when others => NULL;
    end case; 
   when B"110" => 
    case pixel_nr is
     when B"000" => pixel_nr_rmpd := 0;
     when B"001" => pixel_nr_rmpd := 1;
     when B"010" => pixel_nr_rmpd := 2;
     when B"011" => pixel_nr_rmpd := 3;
     when B"100" => pixel_nr_rmpd := 4;
     when others => NULL;
    end case; 
   when others => NULL;    
  end case; 
 return pixel_nr_rmpd;
end function pixel_remap_cl1to6;  

 
end package body trigger_3nn_pkg;
